package org.openxma.dsl.generator.helper;

import org.openxma.dsl.core.model.EnumType;
import org.openxma.dsl.core.model.ValidatorReference;
import org.openxma.dsl.dom.model.Attribute;

public class BasicTypeHelper {
    
    public static BasicType determineBasicType(Attribute attr) {
        String validatorName = BasicTypeHelper.determineRootValidatorName(attr);
        if (ValidatorReference.STRING_VALIDATOR.equals(validatorName)) {
            return BasicType.STRING;
        }
        if (ValidatorReference.NUMBER_VALIDATOR.equals(validatorName)) {
            return BasicType.NUMBER;
        }
        if (ValidatorReference.BOOLEAN_VALIDATOR.equals(validatorName)) {
            return BasicType.BOOLEAN;
        }
        if (ValidatorReference.DATE_VALIDATOR.equals(validatorName)) {
            return BasicType.DATE;
        }
        if (ValidatorReference.TIME_STAMP_VALIDATOR.equals(validatorName)) {
            return BasicType.TIME_STAMP;
        }
        if (ValidatorReference.CUSTOM_VALIDATOR.equals(validatorName)) {
            return BasicType.CUSTOM;
        }
        if (ValidatorReference.I18N_ENUMERATION_VALIDATOR.equals(validatorName)) {
            return BasicType.I18N_ENUMERATION;
        }
        return null;
    }
    
    public static String determineRootValidatorName(Attribute attr) {
        ValidatorReference rootValidatorReference = determineRootValidator(attr);
        if (rootValidatorReference == null) {
            if (attr.getType().getDataType() instanceof EnumType) {
                return ValidatorReference.I18N_ENUMERATION_VALIDATOR;
            }
            throw new RuntimeException("Root validator is not defined for '"+attr.getType().getDataType().getName()+"'");
        }
        return determineRootValidatorName(rootValidatorReference);                
    }

    public static ValidatorReference determineRootValidator(Attribute attr) {
        ValidatorReference validatorReference = attr.getType().getValidatorReferenceDefinition();
        if (validatorReference == null) {
            return null;
        }
        ValidatorReference rootValidatorReference = validatorReference.getRootValidatorReference();
        return rootValidatorReference;
    }
    
    public static String determineRootValidatorName(ValidatorReference rootValidatorReference) {
        String validatorName = rootValidatorReference.getValidator().getName();
        return validatorName;                
    }    
}
