package org.openxma.dsl.generator.helper;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.Stack;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.openxma.dsl.common.extensions.EObjectPropertiesAdapter;
import org.openxma.dsl.core.model.AtomicBoolExpr;
import org.openxma.dsl.core.model.AtomicExpr;
import org.openxma.dsl.core.model.Call;
import org.openxma.dsl.core.model.CondORExpr;
import org.openxma.dsl.core.model.EqualityExpr;
import org.openxma.dsl.core.model.FalseLiteral;
import org.openxma.dsl.core.model.MultiplicativeExpr;
import org.openxma.dsl.core.model.RelationalExpr;
import org.openxma.dsl.core.model.StatusFlag;
import org.openxma.dsl.core.model.TrueLiteral;
import org.openxma.dsl.core.util.NextOperatorSwitch;
import org.openxma.dsl.dom.model.ConditionsBlock;
import org.openxma.dsl.dom.model.Entity;
import org.openxma.dsl.dom.model.Property;
import org.openxma.dsl.pom.model.CheckBox;
import org.openxma.dsl.pom.model.ConditionedLogic;
import org.openxma.dsl.pom.model.DataObjectVariable;
import org.openxma.dsl.pom.model.FieldFlag;
import org.openxma.dsl.pom.model.GuiElement;
import org.openxma.dsl.pom.model.LogicBlock;
import org.openxma.dsl.pom.model.LogicItem;
import org.openxma.dsl.pom.model.ObjectProperty;
import org.openxma.dsl.pom.model.Page;
import org.openxma.xmadsl.GuidesignExtension;
import org.openxma.xmadsl.LinkerUtil;

import at.spardat.xma.guidesign.HiddenWidget;
import at.spardat.xma.guidesign.XMACompProperty;
import at.spardat.xma.guidesign.XMAComponent;
import at.spardat.xma.guidesign.XMAPage;
import at.spardat.xma.guidesign.XMAWidget;
import at.spardat.xma.guidesign.flex.ApplicationFunction;
import at.spardat.xma.guidesign.flex.ApplicationProperty;
import at.spardat.xma.guidesign.flex.BracketExpression;
import at.spardat.xma.guidesign.flex.Expression;
import at.spardat.xma.guidesign.flex.FlexFactory;
import at.spardat.xma.guidesign.flex.IExpVariable;
import at.spardat.xma.guidesign.flex.OperatorExpression;
import at.spardat.xma.guidesign.flex.VariableExpression;
import at.spardat.xma.guidesign.flex.XMAApplicationContext;
import at.spardat.xma.guidesign.flex.XMAStateFlag;
import at.spardat.xma.guidesign.flex.type.OperatorType;
import at.spardat.xma.guidesign.flex.type.VariableAccess;
import at.spardat.xma.guidesign.types.PropertyType;

/**
 * Xtend java extensions for core Expression.
 */
public class ExpressionExtension {
	private static final String NULL_LITERAL = "NULL";
	private static final String OPERATOR_STACK = "operator_stack";
	private static final String DEFAULT_CONDITIONS_PROPERTY = "defaultConditionsProperty";
	private static final String IS_PART_OF_OR_CHAIN_PROPERTY = "isPartOfOrChain";	
	private static final String VARIABLE_EXPRESSION_PROPERTY = "variableExpression";	

	/**
	 * @param eObject
	 * @return the next available operator based on the given object
	 */
	public static String getNextOperator(EObject eObject) {
		return new NextOperatorSwitch().doSwitch(eObject);
	}
	
	/**
	 * Pushes the specified operator onto the top of the stack stored on the root container of the given object.
	 * 
	 * @param eObject the adapter holding the stack
	 * @param operator the operator to push
	 */
	public static void pushOperator(EObject eObject, String operator) {
		EObject rootContainer = EcoreUtil.getRootContainer(eObject);
		Stack<String> stack = (Stack<String>) EObjectPropertiesAdapter.getProperty(rootContainer, OPERATOR_STACK);
		if (stack==null) {
			stack = new Stack<String>();
			EObjectPropertiesAdapter.setProperty(rootContainer, OPERATOR_STACK,stack);
		}
		if (operator!=null && !NULL_LITERAL.equals(operator)) {
			stack.push(operator);
		}
	}
	
	/**
	 * @param eObject the adapter holding the stack
	 * @return the operator at the top of the stack and removes it
	 */
	public static String popOperator(EObject eObject) {
		EObject rootContainer = EcoreUtil.getRootContainer(eObject);
		Stack<String> stack = (Stack<String>) EObjectPropertiesAdapter.getProperty(rootContainer, OPERATOR_STACK);
		if (stack!=null && !stack.isEmpty()) {
			return stack.pop();
		}
		return "";
	}
	
	/**
	 * @param equalityExpr the expression to evaluate
	 * @return true of false whether the given expression evals to a primitive wrapper type
	 */
	public static boolean isPrimitiveWrapperType(EqualityExpr equalityExpr) {
		String typeName = new ExpressionTypeSwitch().doSwitch(equalityExpr);
		Boolean primitiveWrapperType = SimpleTypeExtension
				.isPrimitiveWrapperType(typeName);
		return primitiveWrapperType;
	}
		
	public static ApplicationFunction findApplicationFunction(Call call, XMAComponent xmaComponent) {

		XMAApplicationContext application = GuidesignExtension.getXMAApplicationContext(xmaComponent);
		
		for (Object obj : application.getFunctions()) {
			ApplicationFunction function = (ApplicationFunction)obj;
			if (function.getNamFunction().equals(call.getFunction().getName())) {
				return function;
			}
		}
		return createApplicationFunction(application,call,xmaComponent);		
	}
	
	public static at.spardat.xma.guidesign.flex.type.VariableAccess translateAccessType(EObject object, org.openxma.dsl.core.model.VariableAccess accessType) {		
		switch(accessType.getValue()) {
			case org.openxma.dsl.core.model.VariableAccess.VALUE_VALUE: return at.spardat.xma.guidesign.flex.type.VariableAccess.VALUE_LITERAL;
			case org.openxma.dsl.core.model.VariableAccess.HAS_VALUE_VALUE: return at.spardat.xma.guidesign.flex.type.VariableAccess.HAS_VALUE_LITERAL;
			case org.openxma.dsl.core.model.VariableAccess.GET_SELECTED_VALUE: return at.spardat.xma.guidesign.flex.type.VariableAccess.GET_SELECTED_LITERAL;
			case org.openxma.dsl.core.model.VariableAccess.IS_SELECTED_VALUE: return at.spardat.xma.guidesign.flex.type.VariableAccess.IS_SELECTED_LITERAL;
			case org.openxma.dsl.core.model.VariableAccess.GET_SELECTION_COUNT_VALUE: return at.spardat.xma.guidesign.flex.type.VariableAccess.GET_SELECTION_COUNT_LITERAL;
			case org.openxma.dsl.core.model.VariableAccess.SIZE_VALUE: return at.spardat.xma.guidesign.flex.type.VariableAccess.SIZE_LITERAL;
			case org.openxma.dsl.core.model.VariableAccess.IS_VALID_VALUE: return at.spardat.xma.guidesign.flex.type.VariableAccess.IS_VALID_LITERAL;
			
			default:
				if (object instanceof GuiElement) {
					if (object instanceof org.openxma.dsl.pom.model.RadioButton || object instanceof CheckBox) {
						return at.spardat.xma.guidesign.flex.type.VariableAccess.IS_SELECTED_LITERAL;					
					} 
				} 
				if (object instanceof XMAWidget) {
					if (object instanceof at.spardat.xma.guidesign.RadioButton || object instanceof at.spardat.xma.guidesign.CheckButton) {
						return at.spardat.xma.guidesign.flex.type.VariableAccess.IS_SELECTED_LITERAL;
					}
				} 
				return at.spardat.xma.guidesign.flex.type.VariableAccess.VALUE_LITERAL;							
		}
	}
	
	/*
	public static VariableExpression createXMAVariable(Table table, VariableAccess accessType, XMAComponent comp, XMAPage currPage, DataObjectVariable dov) {
		VariableExpression tableVariable = FlexFactory.eINSTANCE.createVariableExpression();
		List<XMAWidget> generatedWidgets = ComplexTypeExtension.getGeneratedXMAWidgetsForGuiElement(table);		
		if (generatedWidgets == null || generatedWidgets.size() == 0) {
			throw new RuntimeException("The generated XMAWidgets can not be determined for the GuiElement: "+table);
		}		
		XMATable xmaTable = (XMATable)generatedWidgets.get(0);		
		tableVariable.setTypAccess(accessType);
		tableVariable.setSource(xmaTable);	
		return tableVariable;
	}
	
	public static VariableExpression createXMAVariable(Text text, VariableAccess accessType, XMAComponent comp, XMAPage currPage, DataObjectVariable dov) {
		VariableExpression textVariable = FlexFactory.eINSTANCE.createVariableExpression();
		List<XMAWidget> generatedWidgets = ComplexTypeExtension.getGeneratedXMAWidgetsForGuiElement(text);		
		if (generatedWidgets == null || generatedWidgets.size() == 0) {
			throw new RuntimeException("The generated XMAWidgets can not be determined for the GuiElement: "+text);
		}		
		XMAText xmaText = (XMAText)generatedWidgets.get(0);		
		textVariable.setTypAccess(accessType);
		textVariable.setSource(xmaText);	
		return textVariable;
	}
	
	public static VariableExpression createXMAVariable(Combo combo, VariableAccess accessType, XMAComponent comp, XMAPage currPage, DataObjectVariable dov) {
		VariableExpression textVariable = FlexFactory.eINSTANCE.createVariableExpression();
		List<XMAWidget> generatedWidgets = ComplexTypeExtension.getGeneratedXMAWidgetsForGuiElement(combo);		
		if (generatedWidgets == null || generatedWidgets.size() == 0) {
			throw new RuntimeException("The generated XMAWidgets can not be determined for the GuiElement: "+combo);
		}		
		SimpleCombo simpleCombo = (SimpleCombo)generatedWidgets.get(0);		
		textVariable.setTypAccess(accessType);
		textVariable.setSource(simpleCombo);	
		return textVariable;
	}	
	*/
		
	public static VariableExpression createXMAVariableForOtherGuiElement(GuiElement guiElement, VariableAccess accessType, XMAComponent comp, XMAPage currPage, DataObjectVariable dov) {
		VariableExpression textVariable = FlexFactory.eINSTANCE.createVariableExpression();
		List<XMAWidget> generatedWidgets = ComplexTypeExtension.getGeneratedXMAWidgetsForGuiElement(guiElement);		
		if (generatedWidgets == null || generatedWidgets.size() == 0) {
			throw new RuntimeException("The generated XMAWidget can not be determined for the GuiElement: "+guiElement);
		}		
		IExpVariable iExpVariable = (IExpVariable)generatedWidgets.get(0);		
		textVariable.setTypAccess(accessType);
		textVariable.setSource(iExpVariable);	
		return textVariable;
	}			
	
	public static VariableExpression createXMAVariable(Page page, VariableAccess accessType, XMAComponent comp, XMAPage currPage, DataObjectVariable dov) {
		VariableExpression pageVariable = FlexFactory.eINSTANCE.createVariableExpression();			
		XMAPage referencedXMAPage = GuidesignExtension.findXMAPage(comp,page);				
		pageVariable.setTypAccess(accessType);
		pageVariable.setSource(referencedXMAPage);	
		return pageVariable;
	}			
	
	/*
	 * Statusflag - variable
	 */ 
	// TODO use in Extend
	public static VariableExpression createXMAVariable(Property property, VariableAccess accessType, XMAComponent comp, XMAPage currPage, DataObjectVariable dov) {

		VariableExpression stateFlagVariable = FlexFactory.eINSTANCE.createVariableExpression();		
		ApplicationProperty referencedProperty = findApplicationProperty(property,comp,dov);
			
		stateFlagVariable.setTypAccess(accessType);	
		stateFlagVariable.setSource(referencedProperty);
		
		return stateFlagVariable; 	
	}
	
	
	public static VariableExpression createXMAVariable(ObjectProperty property, VariableAccess accessType, XMAComponent comp, XMAPage currPage, DataObjectVariable dov) {

		VariableExpression stateFlagVariable = FlexFactory.eINSTANCE.createVariableExpression();		
		IExpVariable expVariable = findHiddenWidget(currPage,property.getName());
		if (expVariable == null) {
			expVariable = findXMACompProperty(comp,property.getName());			
		}
			
		stateFlagVariable.setTypAccess(accessType);	
		stateFlagVariable.setSource(expVariable);
		
		return stateFlagVariable; 	
	}
	
    public static VariableExpression createXMAVariable(IExpVariable xmaExpVariable, VariableAccess accessType, XMAComponent comp, XMAPage currPage, DataObjectVariable dov) {        
        IExpVariable referencedIExpVariable = getReferencedIExpVariable(comp,xmaExpVariable);
        VariableExpression varExpr = createXMAVariableForReferencedIExpVariable(referencedIExpVariable,accessType);                
        return varExpr;
    }	
	
	private static HiddenWidget findHiddenWidget(XMAPage currPage, String name) {
		if (currPage.getComposite() != null) {
			for (EObject obj : currPage.getComposite().eContents()) {
				if (obj instanceof HiddenWidget && ((HiddenWidget)obj).getNamInstance().equals(name)) {
					return (HiddenWidget)obj;
				}
			}		
		}
		return null;		
	}
	
	private static XMACompProperty findXMACompProperty(XMAComponent comp, String name) {
		for (Object obj : comp.getProperty()) {
			XMACompProperty cp = (XMACompProperty)obj;
			if (cp.getNamInstance().equals(name)) {
				return  cp;
			}
		}
		return null;
	}
	
	// TODO use in Extend (first check and implement used functions)
//	public static XMAStateFlag findXMAStateFlag(StatusFlag statusFlag, XMAComponent comp, XMAPage currPage, DataObjectVariable dov ) {
//
//		EObject stateFlagParent = StatusFlagExtension.getOwnerOfStatusFlag(statusFlag);
//
//		if (stateFlagParent instanceof Component) {
//			return StatusFlagExtension.findXMAStateFlagForComponent(comp,statusFlag,dov);
//		} else if (stateFlagParent instanceof Page || stateFlagParent instanceof Entity) {
//			return StatusFlagExtension.findXMAStateFlagForPage(currPage,statusFlag,comp,dov);
//		} else if (stateFlagParent instanceof ApplicationSession) {
//			return StatusFlagExtension.findXMAStateFlagForApplication(comp,statusFlag,dov);
//		}
//		throw new RuntimeException("XMAStateFlag not found "+statusFlag.getName());
//	}
	

	// TODO use in Extend
	public static ApplicationProperty findApplicationProperty(Property property, XMAComponent xmaComponent, DataObjectVariable dov) {
		XMAApplicationContext application = GuidesignExtension.getXMAApplicationContext(xmaComponent);
		String propName = property.getName();		
		for (Object obj : application.getProperties()) {
			ApplicationProperty appProp = (ApplicationProperty)obj;
			if (appProp.getNamProperty().equals(propName)) {
				return appProp;
			}
		}
		return createApplicationProperty(application,property,xmaComponent);
	}
	
	/**
	 * Mapping for supported property-types
	 */		
	public static PropertyType getXMAPropertyType(Property property) {
		// TODO replace this dummy implementation with something else which
		// allows a save and consistent type determination
		String typeName = property.getType().getName();		
		if ("String".equals(typeName)) return PropertyType.STRING_LITERAL;
		if ("Boolean".equals(typeName))  return PropertyType.BOOLEAN_LITERAL;
		if ("Date".equals(typeName))  return PropertyType.DATE_LITERAL;
		if ("Integer".equals(typeName))  return PropertyType.INT_LITERAL;
		if ("Number".equals(typeName))  return PropertyType.DOUBLE_LITERAL;		// TODO: discuss rename to "Decimal"	
		return PropertyType.STRING_LITERAL;	
	}
	
	public static ApplicationProperty createApplicationProperty(XMAApplicationContext application, Property property, XMAComponent xmaComponent) {
		
		ApplicationProperty appProperty = FlexFactory.eINSTANCE.createApplicationProperty();
		
		appProperty.setTypProperty(getXMAPropertyType(property));
		appProperty.setNamProperty(property.getName());
		
		application.getProperties().add(appProperty);
			
		return appProperty;	
	}
		
	public static ApplicationFunction createApplicationFunction(XMAApplicationContext application, Call call, XMAComponent xmaComponent) {		
		ApplicationFunction appFunction = FlexFactory.eINSTANCE.createApplicationFunction();	
		appFunction.setNamFunction(call.getFunction().getName());		
		application.getFunctions().add(appFunction);			
		return appFunction;
	}
	
	public static Expression cloneExpression(Expression exp) {
		return (Expression)EcoreUtil.copy(exp);
	}
	
	/**
	 * Determine the boolean literal of a EqualityExpr.
	 * In case the expression is not a boolean literal, the
	 * method returns null.
	 * 
	 * @param expression
	 * @return
	 */
	public static Boolean booleanLiteralFor(EqualityExpr expression) {
		if (expression.getRight() == null) {
			CondORExpr orExpr = expression.getLeft();
			if (orExpr.getRights().size() == 0 && orExpr.getLeft().getRights().size() == 0) {
				AtomicBoolExpr boolExpr = orExpr.getLeft().getLeft();
				if (boolExpr instanceof RelationalExpr) {
					RelationalExpr re = (RelationalExpr)boolExpr;
					if (re.getRight() == null && re.getLeft().getRights().size() == 0) {
						MultiplicativeExpr me = re.getLeft().getLeft();
						if (me.getRights().size() == 0) {
							AtomicExpr ae = me.getLeft();
							if (ae instanceof TrueLiteral) {
								return true;
							} else if (ae instanceof FalseLiteral) {
								return false;
							} else {
								return null;
							}
						}
					}
				}					
			}
		}
		return null;
	}
	
	public static void putDefaultCondition(GuiElement guiElement, List<VariableExpression> variableExpressions, List<ConditionedLogic> conditionedLogics, Object obj) {	    
	    if (variableExpressions.size()>0) {
	        // merge and negate expressions
            Expression joinedExpression = null;	        
	        for (VariableExpression varExpr : variableExpressions) {
	            if (varExpr != null) {
                    Expression negatedExpr = not((Expression)EcoreUtil.copy(varExpr)); 
                    if (joinedExpression == null) {
                        joinedExpression = negatedExpr;
                    } else {
                        joinedExpression = and(joinedExpression,negatedExpr);
                    }	            	  
	            }
	        }	        
            if (joinedExpression != null) {                                
                List<XMAWidget> generatedWidgets = ComplexTypeExtension.getGeneratedXMAWidgetsForGuiElement(guiElement);
    	        for (XMAWidget widget : generatedWidgets) {
    	            Object conditionsProperty = EObjectPropertiesAdapter.getProperty(widget,DEFAULT_CONDITIONS_PROPERTY);
    	            DefaultConditions dc = null;
    	            if (conditionsProperty instanceof DefaultConditions) {
    	                dc = (DefaultConditions)conditionsProperty;
    	            } else {
    	                dc = new DefaultConditions();
    	                EObjectPropertiesAdapter.setProperty(widget,DEFAULT_CONDITIONS_PROPERTY,dc);    	                
    	            }
    	            dc.conditionsMap.put(obj,bracket((Expression)EcoreUtil.copy(joinedExpression)));
    	            if (conditionedLogics.size()>0) {
    	                dc.conditionedLogicMap.put(obj,conditionedLogics);
    	            }    	            
    	        }
	        }
	    }	    	   
	}
	
	public static Object getDefaultConditions(XMAWidget xmaWidget) {
	    return EObjectPropertiesAdapter.getProperty(xmaWidget,DEFAULT_CONDITIONS_PROPERTY);	    
	}
	
    public static Expression getDefaultCondition(XMAWidget widget, Object obj) {        
        Object conditionsProperty = getDefaultConditions(widget);
        if (conditionsProperty instanceof DefaultConditions) {
            DefaultConditions defaultConditions = (DefaultConditions)conditionsProperty;
            Object expr = defaultConditions.conditionsMap.get(obj);
            if (expr instanceof Expression) {
                return (Expression)expr;
            }                        
        }
        return null;
    }
    
    public static void removeDefaultConditions(GuiElement guiElement) {
        List<XMAWidget> generatedWidgets = ComplexTypeExtension.getGeneratedXMAWidgetsForGuiElement(guiElement);
        for (XMAWidget widget : generatedWidgets) {        
            EObjectPropertiesAdapter.removeProperty(widget,DEFAULT_CONDITIONS_PROPERTY);
        }
    }
    
    static class DefaultConditions {
        HashMap conditionedLogicMap = new HashMap();        
        HashMap conditionsMap = new HashMap();        
    }
    
    /**
     * Bracket-expression
     */ 
    public static Expression bracket(Expression expr) {        
        if (expr != null) {
            BracketExpression bracketExpression = FlexFactory.eINSTANCE.createBracketExpression();
            bracketExpression.setSubExpression(expr);
            return bracketExpression;
        } else {
            return null;
        }
    }
    
    /**
     * Force bracket expression in case expr is not a bracket expression.
     */     
    public static Expression forceBracket(Expression expr) {
        if (expr instanceof BracketExpression) {
            return expr;
        } else {
            return bracket(expr);
        }
    }
    
    /**
     * Encapsulates expr with a bracket when needed to ensure
     * the correct interpretation of the term.
     * 
     * @param expr
     * @return
     */
    public static Expression encapsulateWithBracketWhenNeeded(Expression expr) {
        if (expr instanceof BracketExpression) {
            return expr;
        } else if (expr instanceof VariableExpression) {
           return expr;  
        } else {
            return bracket(expr);
        }
    }      
    
    /**
     * Negate expression
     */ 
    public static Expression not(Expression expr) {
        if (expr != null) {
            OperatorExpression notExpr =  FlexFactory.eINSTANCE.createOperatorExpression();
            notExpr.setTypOperator(OperatorType.NOT_LITERAL);            
            notExpr.setRightExpression(encapsulateWithBracketWhenNeeded(expr)); 
            return notExpr;
        } else {
            return null;
        }
    }
    
    /**
     * Join two XMA-expressions with an operator
     */
    public static Expression join(OperatorType operator, Expression expr1, Expression expr2) {
        if (expr1 != null) {
            if (expr2 != null) {
                OperatorExpression newExpr =  FlexFactory.eINSTANCE.createOperatorExpression();
                newExpr.setTypOperator(operator);
                newExpr.setLeftExpression(expr1);
                newExpr.setRightExpression(expr2);
                return newExpr;                                
            } else {
                return expr1;
            }
        } else {
            return expr2;
        }
    }
    
    /**
     * And-expression
     */
    public static Expression and(Expression expr1, Expression expr2) {       
        return join(OperatorType.AND_LITERAL,expr1,expr2);
    }
    
    /**
     * Or-expression
     */
    public static Expression or(Expression expr1, Expression expr2) {       
        return join(OperatorType.OR_LITERAL,expr1,expr2);
    }    
    
    /**
     * Build the expression to be set considering the conditions in the logic block. 
     */         
    public static Expression concatenateLogicConditions(Expression expr, Object defaultConditionsObj, Object flag, org.openxma.dsl.pom.model.ConditionedLogic conditionedLogic, Expression previousExpression, Boolean defaultValue) {
        DefaultConditions defaultConditions = (DefaultConditions)defaultConditionsObj;
        Expression defaultCondition = (defaultConditions != null) ? (Expression)defaultConditions.conditionsMap.get(flag) : null;              
        /* Do nothing in case of an empty expression (should never happen) */
        if (expr == null) {
            return null;
        }       
        /* Not in a case block (default block or inline definition) */
        if (conditionedLogic == null) {
            if (defaultCondition != null) {
                return and(cloneExpression(defaultCondition),forceBracket(expr));
            } else {
                return expr;
            }
        }        
        /* Case block */        
        // Special treatment in case we are in a case block, but the previous value
        // is null. This means that the flag to which this expression belongs        
        // is not defined in the "default" block. Therefore the default expression is
        // used instead the previous value. 
        // The default expression is determined from the default value and the default
        // condition.
        if (previousExpression == null && defaultCondition != null) {
            if (defaultValue == false) {
                previousExpression = and(forceBracket(defaultCondition),ValidatorExtension.createBooleanLiteral(defaultValue));
            } else {
                previousExpression = defaultCondition; 
            }
        }
        // Determine the condition which expresses when this case should be applied 
        Expression caseCondition = getVariableExpression(conditionedLogic);               
        Expression followingCaseConditions = getFollowingCaseConditions(flag, conditionedLogic, defaultConditions);
        Expression caseConditionWithoutFollowingCaseConditions = and(caseCondition,not(followingCaseConditions));
        // Combine the previous expression with the case condition
        Expression exprWithCase =  and(cloneExpression(caseConditionWithoutFollowingCaseConditions),expr);        
        if (previousExpression != null) { 
            // Avoid brackets which are not needed by chaining or expressions
            // Generate: (A || B || C) instead of (((A || B) || C)) 
            if (EObjectPropertiesAdapter.getProperty(previousExpression,IS_PART_OF_OR_CHAIN_PROPERTY) == null) {
                previousExpression = forceBracket(previousExpression);                
            }
            Expression resultingExpression = or(previousExpression,bracket(exprWithCase));
            EObjectPropertiesAdapter.setProperty(resultingExpression,IS_PART_OF_OR_CHAIN_PROPERTY,true);                    
            return resultingExpression;
        } else {
            return exprWithCase; // Should never happen
        }
    }

    private static Expression getFollowingCaseConditions(Object flag, org.openxma.dsl.pom.model.ConditionedLogic conditionedLogic, DefaultConditions defaultConditions) {
        List<Expression> followingCaseExpressions = new ArrayList<Expression>();        
        List<ConditionedLogic> conditionedLogics = (List<ConditionedLogic>)defaultConditions.conditionedLogicMap.get(flag);        
        boolean addToList = false;
        for (int i=0;i<conditionedLogics.size();i++) {
            ConditionedLogic cl = conditionedLogics.get(i); 
            if (addToList) {
                followingCaseExpressions.add(getVariableExpression(cl));
            }
            if (cl == conditionedLogic) {                  
                addToList = true;
                continue;
            } 
        }
        return concatinateAllByOr(followingCaseExpressions);
    }
    
    public static List<GuiElement> collectAllGuiElementsInLogic(LogicBlock logicBlock) {
        Set<GuiElement> guiElements = new HashSet<GuiElement>();
        for (LogicItem logicItem : logicBlock.getLogicItem()) {
            guiElements.add(logicItem.getGuiElement());
        }
        for (ConditionedLogic conditionedLogic : logicBlock.getConditionedLogic()) {
            for (LogicItem logicItem : conditionedLogic.getLogicItem()) {
                guiElements.add(logicItem.getGuiElement());
            }        
        }
        List<GuiElement> result = new ArrayList<GuiElement>();
        result.addAll(guiElements);
        return result;
    }
    
    public static List<StatusFlag> collectStatusFlagsForConditions(GuiElement guiElement, List<ConditionedLogic> conditionedLogicBlocks, Class flagClass) {        
        List<StatusFlag> variableExpressions = new ArrayList<StatusFlag>();
        for (ConditionedLogic conditionedLogic : conditionedLogicBlocks) {
            for (LogicItem logicItem : conditionedLogic.getLogicItem()) {
                if (logicItem.getGuiElement() == guiElement) {
                    boolean found = false;                    
                    for (FieldFlag fieldFlag : logicItem.getFieldFlags()) {                       
                        if (flagClass.isInstance(fieldFlag)) {
                            found = true;
                            break;
                        }
                    }
                    if (found) {
                        variableExpressions.add(conditionedLogic.getStatusFlag());
                        break;
                    }
                }
            }            
        }
        return variableExpressions;
    }    
    
    public static Expression concatinateAllByOr(List<Expression> expressions) {
        if (expressions == null || expressions.size()==0) {
            return null;
        }
        Expression expr = expressions.get(0);        
        for (int i=1;i<expressions.size();i++) {
            Expression expr2 = expressions.get(i);
            expr=or(expr,expr2);                        
        }
        return expr;
    }   
    
    public static VariableExpression createXMAVariableForReferencedIExpVariable(IExpVariable referencedIExpVariable, VariableAccess accessType) {        
        VariableExpression expVariable = FlexFactory.eINSTANCE.createVariableExpression();
        expVariable.setTypAccess(accessType);  
        expVariable.setSource(referencedIExpVariable); 
        return expVariable;    
    }
    
    public static IExpVariable getReferencedIExpVariable(EObject xmaComp, IExpVariable xmaExpVariable) {
        List<EObject> objectPath = LinkerUtil.getModelObjectPath(xmaExpVariable);
        EObject referencedObj = LinkerUtil.findXMAPathElement(xmaComp,objectPath);        
        return (IExpVariable)referencedObj;                
    }
    
    public static EObject getOwnerOfStatusFlag(StatusFlag statusFlag) {
        ConditionsBlock conditionsBlock = (ConditionsBlock)statusFlag.eContainer();   
        return conditionsBlock.eContainer();        
    }
    
    public static String getXMAStateFlagName(StatusFlag statusFlag, DataObjectVariable dov) {
        EObject ownerOfStatusFlag = getOwnerOfStatusFlag(statusFlag);          
        if (ownerOfStatusFlag instanceof Entity) {
           return "st_"+dov.getName()+"_"+statusFlag.getName();
        } else {
           return "st_"+statusFlag.getName();
        }    
    }
    
    public static XMAStateFlag findXMAStateFlagInList(List<XMAStateFlag> xmaStateFlagList, StatusFlag statusFlag, DataObjectVariable dov) {
        String namXMAStateFlag = getXMAStateFlagName(statusFlag,dov);
        for (XMAStateFlag xmaStateFlag : xmaStateFlagList) {
            if (xmaStateFlag.getNamState().equals(namXMAStateFlag)) {
                return xmaStateFlag;
            }
        }
        return null;
    }
    
    public static void putVariablesForConditionedLogic(List<ConditionedLogic> conditionedLogics, List<VariableExpression> variables) {
        if (conditionedLogics == null) {
            return;
        }
        if (conditionedLogics.size() != variables.size()) {
            throw new IllegalArgumentException("Size of variables must be equal to size of conditionedLogics");
        }
        for (int i=0;i<conditionedLogics.size();i++) {
            EObjectPropertiesAdapter.setProperty(conditionedLogics.get(i),VARIABLE_EXPRESSION_PROPERTY,variables.get(i));                                                          
        }        
    }
    
    public static VariableExpression getVariableExpression(ConditionedLogic conditionedLogic) {
        Object obj = EObjectPropertiesAdapter.getProperty(conditionedLogic,VARIABLE_EXPRESSION_PROPERTY);
        if (obj instanceof VariableExpression) {
            return (VariableExpression)obj;
        }
        throw new RuntimeException("The property 'variableExpression' is not of type VariableExpression");
    }

}

