package org.openxma.dsl.generator.impl;

import java.util.Collection;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.EStructuralFeature.Setting;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.openxma.dsl.common.DslConstants;
import org.openxma.dsl.core.model.Model;
import org.openxma.dsl.pom.PomPackage;

import at.spardat.xma.guidesign.XMAComponent;
import at.spardat.xma.guidesign.XMAPage;
import at.spardat.xma.guidesign.flex.XMAFunction;
import at.spardat.xma.guidesign.generate.ComponentLogicAttachementStrategy;
import at.spardat.xma.guidesign.generate.PageLogicAttachementStrategy;

/**
 * Convenience base class for component- and pagelogic attachment strategies
 * which provides some support methods to to load and access pom model files.
 */
public abstract class AbstractPomDslAttachmentStrategySupport implements ComponentLogicAttachementStrategy,
        PageLogicAttachementStrategy {

    protected static Model getReferencingPomModel(EObject xmaObject) {
        Collection<EStructuralFeature.Setting> settings = EcoreUtil.UsageCrossReferencer.find(xmaObject, xmaObject
                .eResource().getResourceSet());
        for (Setting setting : settings) {
            if (setting.getEObject().eClass().getEPackage().equals(PomPackage.eINSTANCE)) {
                return (Model) settings.iterator().next().getEObject().eResource().getEObject("/");
            }
        }
        ResourceSet resourceSet = xmaObject.eResource().getResourceSet();
        EList<Resource> resources = resourceSet.getResources();
        for (Resource resource : resources) {
            if (DslConstants.OPENXMA_POM_FILE_EXTENSION.equals(resource.getURI().fileExtension())
                    && resource.getURI().trimFileExtension().lastSegment().equals(
                            xmaObject.eResource().getURI().trimFileExtension().lastSegment())) {
                return (Model) resource.getEObject("/");
            }
        }
        return null;
    }

    public String genAdditionalConstructorCodeClient(XMAComponent component) {
        return "";
    }

    public String genAdditionalConstructorCodeServer(XMAComponent component) {
        return "";
    }

    public String genAdditionalImportsClient(XMAComponent component) {
        return "";
    }

    public String genAdditionalImportsServer(XMAComponent component) {
        return "";
    }

    public String genAdditionalMemberVariablesClient(XMAComponent component) {
        return "";
    }

    public String genAdditionalMemberVariablesServer(XMAComponent component) {
        return "";
    }

    public String genAdditionalMethodsClient(XMAComponent component) {
        return "";
    }

    public String genAdditionalMethodsServer(XMAComponent component) {
        return "";
    }

    public String genCommandImplementationClient(XMAComponent component, XMAFunction function) {
        return "";
    }

    public String genCommandImplementationServer(XMAComponent component, XMAFunction function) {
        return "";
    }

    public String genAdditionalConstructorCodeClient(XMAPage page) {
        return "";
    }

    public String genAdditionalConstructorCodeServer(XMAPage page) {
        return "";
    }

    public String genAdditionalImportsClient(XMAPage page) {
        return "";
    }

    public String genAdditionalImportsServer(XMAPage page) {
        return "";
    }

    public String genAdditionalMemberVariablesClient(XMAPage page) {
        return "";
    }

    public String genAdditionalMemberVariablesServer(XMAPage page) {
        return "";
    }

    public String genAdditionalMethodsClient(XMAPage page) {
        return "";
    }

    public String genAdditionalMethodsServer(XMAPage page) {
        return "";
    }

    public String genCommandImplementationClient(XMAPage page, XMAFunction function) {
        return "";
    }

    public String genCommandImplementationServer(XMAPage page, XMAFunction function) {
        return "";
    }

    public String genAdditionalClientEventCodeClient(XMAPage page, boolean previousEventsExists) {
        return "";
    }

    public String genAdditionalCreateWidgetsCodeClient(XMAPage page) {
        return "";
    }

    public String genAdditionalRemoveWidgetsCodeClient(XMAPage page) {
        return "";
    }

    public String genAdditionalEnterCodeClient(XMAPage page) {
        return "";
    }
    
    public String genAdditionalStateChangedClient(XMAPage page) {
        return "";
    }    
    
    public String genAdditionalResources(XMAPage page) {
        return "";
    }    
}
